# import requests
#
# from typing import Union
#
# from mbnk.decorators import request, post_request
# from mbnk.responses import CurrencyRatesResponse
# from mbnk.responses import ClientInfoResponse
# from mbnk.responses import StatementResponse
#
# from mbnk.instances import CurrencyListItem
# from mbnk.instances import Account
# from mbnk.instances import Jar
# from mbnk.instances import Transaction
#
# from mbnk.exceptions import MonobankBaseException
#
# from mbnk.utils import is_exception
#
#
# class PublicData:
#     def __init__(
#         self,
#         base_url: str,
#         headers
#     ):
#         self.base__url = base_url
#         self.headers = headers
#
#         self.currency_rates_url = f"{self.base__url}/bank/currency"
#
#     @request(url='currency_rates_url')
#     def currency_rates(self, response_data) -> Union[CurrencyRatesResponse, MonobankBaseException]:
#
#         return CurrencyRatesResponse(
#             list=[
#                 CurrencyListItem(
#                     currency_code_a=int(response_item['currencyCodeA']),
#                     currency_code_b=int(response_item['currencyCodeB']),
#                     date=response_item['date'],
#                     rate_sell=response_item['rateSell'],
#                     rate_buy=response_item['rateBuy'],
#                     rate_cross=response_item['rateCross']
#                 ) for response_item in response_data
#             ]
#         )
#
#
# class Personal:
#     def __init__(
#         self,
#         base_url: str,
#         headers
#     ):
#         self.base__url = base_url
#         self.headers = headers
#
#         self.client_info_url = f"{self.base__url}/personal/client-info"
#         self.set_webhook_url = f"{self.base__url}/personal/webhook"
#         self.statement_url = self.base__url + "/personal/statement/{account}/{from_date}"
#
#     @request(url='client_info_url')
#     def info(self, response_data):
#
#         return ClientInfoResponse(
#             client_id=response_data['clientId'],
#             name=response_data['name'],
#             web_hook_url=response_data['webHookUrl'],
#             permissions=response_data['permissions'],
#             accounts=[
#                 Account(
#                     id=account['id'],
#                     send_id=account['sendId'],
#                     balance=int(account['balance']),
#                     credit_limit=int(account['creditLimit']),
#                     type=account['type'],
#                     currency_code=int(account['currencyCode']),
#                     cashback_type=account['cashbackType'] if 'cashbackType' in account else None,
#                     masked_pan=account['maskedPan'],
#                     iban=account['iban'],
#                 ) for account in response_data['accounts']
#             ],
#             jars=[
#                 Jar(
#                     id=jar['id'],
#                     send_id=jar['sendId'],
#                     title=jar['title'],
#                     description=jar['description'],
#                     currency_code=int(jar['currencyCode']),
#                     balance=int(jar['balance']),
#                     goal=int(jar['goal'])
#                 ) for jar in response_data['jars']
#             ]
#         )
#
#     @post_request
#     def set_webhook(self, web_hook_url: str):
#         pass
#
#     def statement(self, from_date: int, to_date: int = None, account: str = 0):
#         print(self.statement_url.format(
#                 account=account,
#                 from_date=from_date,
#                 # to_date=to_date if to_date else '',
#             ))
#         response = requests.get(
#             url=self.statement_url.format(
#                 account=account,
#                 from_date=from_date,
#                 to_date=to_date,
#             ),
#             headers=self.headers
#         )
#
#         response_data = response.json()
#
#         if is_exception(response):
#             return MonobankBaseException(
#                 err_description=response_data['errorDescription']
#             )
#
#         return StatementResponse(
#             list=[
#                 Transaction(
#                     id=transaction['id'],
#                     time=transaction['time'],
#                     description=transaction['description'],
#                     mcc=transaction['mcc'],
#                     original_mcc=transaction['originalMcc'],
#                     hold=transaction['hold'],
#                     amount=transaction['amount'],
#                     operation_amount=transaction['operationAmount'],
#                     currency_code=transaction['currencyCode'],
#                     commission_rate=transaction['commissionRate'],
#                     cashback_amount=transaction['cashbackAmount'],
#                     balance=transaction['balance'],
#                     comment=transaction['comment'] if 'comment' in transaction else None,
#                     receipt_id=transaction['receiptId'] if 'receiptId' in transaction else None,
#                     invoice_id=transaction['invoiceId'] if 'invoiceId' in transaction else None,
#                     counter_edrpou=transaction['counterEdrpou'] if 'counterEdrpou' in transaction else None,
#                     counter_iban=transaction['counterIban'] if 'counterIban' in transaction else None,
#                     counter_name=transaction['counterName'] if 'counterName' in transaction else None,
#                 ) for transaction in response_data
#             ]
#         )
#
#
# class Monobank:
#
#     __base_url__ = "https://api.monobank.ua"
#     __api_token__ = None
#     __headers__ = {}
#
#     def __init__(self, api_token):
#         self.__api_token__ = api_token
#         self.__headers__["X-Token"] = self.__api_token__
#
#         self.public = PublicData(
#             base_url=self.__base_url__,
#             headers=self.__headers__
#         )
#         self.personal = Personal(
#             base_url=self.__base_url__,
#             headers=self.__headers__
#         )
#
#
# class MonoPay:
#
#     __base_url__ = "https://api.monobank.ua"
#     __api_token__ = None
#     __headers__ = {}
#
#     def __init__(self, api_token):
#         self.__api_token__ = api_token
#         self.__headers__["X-Token"] = self.__api_token__
#
#         self.merchant = None
#
#         self.invoice = None
#
#         self.qr = None
#
#         self.wallet = None



from typing import List, Optional
from dataclasses import dataclass


@dataclass
class CurrencyListItem:
    currency_code_a: int
    currency_code_b: int
    date: int
    rate_sell: float
    rate_buy: float
    rate_cross: float


@dataclass
class Account:
    id: str
    send_id: str
    balance: int
    credit_limit: int
    type: str
    currency_code: int
    cashback_type: Optional[str]
    masked_pan: List[str]
    iban: str


@dataclass
class Jar:
    id: str
    send_id: str
    title: str
    description: str
    currency_code: int
    balance: int
    goal: int


@dataclass
class Transaction:
    id: str
    time: int
    description: str
    mcc: int
    original_mcc: int
    hold: bool
    amount: int
    operation_amount: int
    currency_code: int
    commission_rate: int
    cashback_amount: int
    balance: int
    comment: Optional[str]
    receipt_id: Optional[str]
    invoice_id: Optional[str]
    counter_edrpou: Optional[str]
    counter_iban: Optional[str]
    counter_name: Optional[str]



from enum import Enum


class MonoPayApiUrls(str, Enum):
    merchant_details = "{base_url}/api/merchant/details"
    merchant_statement = "{base_url}/api/merchant/statement"
    merchant_pubkey = "{base_url}/api/merchant/pubkey"

    invoice_create = "{base_url}/api/merchant/invoice/create"
    invoice_split = "{base_url}/api/merchant/invoice/split-payments"
    invoice_cancel = "{base_url}/api/merchant/invoice/cancel"
    invoice_status = "{base_url}/api/merchant/invoice/status"
    invoice_invalidation = "{base_url}/api/merchant/invoice/remove"
    invoice_info = "{base_url}/api/merchant/invoice/payment-info"
    invoice_finalize = "{base_url}/api/merchant/invoice/finalize"

    qr_list = "{base_url}/api/merchant/qr/list"
    qr_details = "{base_url}/api/merchant/qr/details"
    qr_reset_amount = "{base_url}/api/merchant/qr/reset-amount"

    wallet_cards = "{base_url}/api/merchant/wallet"
    wallet_payment = "{base_url}/api/merchant/wallet/payment"
    wallet_delete_card = "{base_url}/api/merchant/wallet/card"



from typing import List
from dataclasses import dataclass
from mbnk.instances import CurrencyListItem
from mbnk.instances import Account
from mbnk.instances import Jar
from mbnk.instances import Transaction


@dataclass
class EmptyResponse:
    pass


@dataclass
class CurrencyRatesResponse:
    list: List[CurrencyListItem]


@dataclass
class ClientInfoResponse:
    client_id: str
    name: str
    web_hook_url: str
    permissions: str
    accounts: List[Account]
    jars: List[Jar]


@dataclass
class StatementResponse:
    list: List[Transaction]



import aiohttp
import json
from mbnk.utils import data_builder


def async_get_request(url):
    def outer(func):
        async def inner(self, *args, **kwargs):
            params = data_builder(**kwargs)

            async with aiohttp.ClientSession() as session:
                async with session.get(
                    url=url.format(
                        base_url=self.__base_url__
                    ),
                    headers=self.__headers__,
                    params=params
                ) as response:
                    response_data = await response.json()

                    return func(self, *args, **kwargs, response_data=response_data)

        return inner

    return outer


def async_post_request(url):
    def outer(func):
        async def inner(self, *args, **kwargs):
            data = data_builder(**kwargs)

            async with aiohttp.ClientSession() as session:
                async with session.post(
                    url=url.format(
                        base_url=self.__base_url__
                    ),
                    headers=self.__headers__,
                    data=json.dumps(data)
                ) as response:
                    response_data = await response.json()

                    return func(self, *args, **kwargs, response_data=response_data)

        return inner

    return outer


def async_delete_request(url):
    def outer(func):
        async def inner(self, *args, **kwargs):
            data = data_builder(**kwargs)

            async with aiohttp.ClientSession() as session:
                async with session.delete(
                    url=url.format(
                        base_url=self.__base_url__
                    ),
                    headers=self.__headers__,
                    params=data
                ) as response:
                    response_data = await response.json()

                    return func(self, *args, **kwargs, response_data=response_data)

        return inner

    return outer



from typing import Union, Optional
from mbnk.decorators import (
    async_get_request,
    async_post_request,
    async_delete_request
)
from mbnk.enums import MonoPayApiUrls


class Monobank:
    pass


class Merchant:

    __base_url__ = None
    __headers__ = {}

    def __init__(
            self,
            base_url: str,
            headers: dict
    ):
        self.__base_url__ = base_url
        self.__headers__ = headers

    @async_get_request(url=MonoPayApiUrls.merchant_details)
    def details(self, **kwargs):
        return kwargs["response_data"]

    @async_get_request(url=MonoPayApiUrls.merchant_statement)
    def statement(self, **kwargs):
        return kwargs["response_data"]

    @async_get_request(url=MonoPayApiUrls.merchant_pubkey)
    def pubkey(self, **kwargs):
        return kwargs["response_data"]


class Invoice:

    __base_url__ = None
    __headers__ = {}

    def __init__(
            self,
            base_url: str,
            headers: dict
    ):
        self.__base_url__ = base_url
        self.__headers__ = headers

    @async_post_request(url=MonoPayApiUrls.invoice_create)
    def create(
            self,
            amount: int,
            ccy: Optional[int] = None,
            merchant_paym_info: Optional = None,
            redirect_url: Optional[str] = None,
            web_hook_url: Optional[str] = None,
            validity: Optional[int] = None,
            payment_type: Optional[str] = None,
            qr_id: Optional = None,
            save_card_data: Optional = None,
            **kwargs
    ):
        """
        :param amount:
        :param ccy:
        :param merchant_paym_info:
        :param redirect_url:
        :param web_hook_url:
        :param validity:
        :param payment_type:
        :param qr_id:
        :param save_card_data:
        :return:
        """

        return kwargs["response_data"]

    @async_post_request(url=MonoPayApiUrls.invoice_split)
    def split(
            self,
            invoice_id: str,
            **kwargs
    ):
        return kwargs["response_data"]

    @async_post_request(url=MonoPayApiUrls.invoice_cancel)
    def cancel(
            self,
            invoice_id: str,
            ext_ref: str = None,
            amount: int = None,
            items=None,
            **kwargs
    ):
        return kwargs["response_data"]

    async def status(
            self,
            invoice_id: str,
            **kwargs
    ):
        return kwargs["response_data"]

    @async_post_request(url=MonoPayApiUrls.invoice_invalidation)
    async def invalidation(
            self,
            invoice_id: str,
            **kwargs
    ):
        return kwargs['response_data']

    async def info(
            self,
            invoice_id: str,
            **kwargs
    ):
        return kwargs['response_data']

    async def finalize(
            self,
            invoice_id: str,
            amount: int,
            **kwargs
    ):
        return kwargs['response_data']


class Qr:

    __base_url__ = None
    __headers__ = {}

    def __init__(
            self,
            base_url: str,
            headers: dict
    ):
        self.__base_url__ = base_url
        self.__headers__ = headers

    @async_get_request(url=MonoPayApiUrls.qr_list)
    def list(self, **kwargs):
        return kwargs["response_data"]

    @async_post_request(url=MonoPayApiUrls.qr_details)
    def details(
            self,
            qr_id: str,
            **kwargs
    ):
        """

        :param qr_id:
        :return:
        """
        return kwargs["response_data"]

    @async_post_request(url=MonoPayApiUrls.qr_reset_amount)
    def reset_amount(
            self,
            qr_id: str,
            **kwargs
    ):
        """

        :param qr_id:
        :return:
        """
        return kwargs["response_data"]


class Wallet:

    __base_url__ = None
    __headers__ = {}

    def __init__(
            self,
            base_url: str,
            headers: dict
    ):
        self.__base_url__ = base_url
        self.__headers__ = headers

    @async_get_request(url=MonoPayApiUrls.wallet_cards)
    async def cards(
            self,
            wallet_id: str,
            **kwargs
    ):
        return kwargs["response_data"]

    async def payment(self):
        pass

    @async_delete_request(url=MonoPayApiUrls.wallet_delete_card)
    async def delete_card(
            self,
            card_token: str,
            **kwargs
    ):
        return kwargs["response_data"]


class AsyncMonoPay:

    __base_url__ = "https://api.monobank.ua"
    __api_token__ = None
    __headers__ = {}

    def __init__(self, api_token: str):
        self.__api_token__ = api_token
        self.__headers__["X-Token"] = self.__api_token__

        self.merchant: Merchant = Merchant(
            base_url=self.__base_url__,
            headers=self.__headers__
        )

        self.invoice: Invoice = Invoice(
            base_url=self.__base_url__,
            headers=self.__headers__
        )

        self.qr: Qr = Qr(
            base_url=self.__base_url__,
            headers=self.__headers__
        )

        self.wallet: Wallet = Wallet(
            base_url=self.__base_url__,
            headers=self.__headers__
        )

    def get_api_token(self):
        return self.__api_token__
